@@ -16,7 +16,7 @@ from argostranslate import package
 package.update_package_index()
 
 # Load available packages from local package index
-available_packages = package.load_available_packages()
+available_packages = package.get_available_packages()
 
 # Download and install all available packages
 for available_package in available_packages:
@@ -40,13 +40,13 @@ class IPackage:
 
     Packages are a zip archive of a directory with metadata.json
     in its root the .argosmodel file extension. By default a 
-    OpenNMT CTranslate directory named model/ created using 
+    OpenNMT CTranslate2 directory named model/ created using 
     ct2-opennmt-tf-converter is expected in the root directory
     along with a sentencepiece model named sentencepiece.model
     for tokenizing and Stanza for sentence boundary detection. 
     Packages may also optionally have a README.md in the root.
 
-    from_code and to_code should be ISO 639-1 codes if applicable.
+    from_code and to_code should be ISO 639 codes if applicable.
 
     Example metadata.json
     {
@@ -84,16 +84,23 @@ class IPackage:
                 if README.md can't be read
 
         """
-        readme_path = self.package_path / 'README.md'
-        if not readme_path.is_file():
-            return None
-        with open(readme_path, 'r') as readme_file:
-            return readme_file.read()
-        return None
+        raise NotImplementedError()
+
+    def get_description(self):
+        raise NotImplementedError()
+
+    def __eq__(self, other):
+        return self.package_version == other.package_version and \
+                self.argos_version == other.argos_version and \
+                self.from_code == other.from_code and \
+                self.from_name == other.from_name and \
+                self.to_code == other.to_code and \
+                self.to_name == other.to_name
 
     def __str__(self):
         return "{} -> {}".format(self.from_name, self.to_name)
 
+
 class Package(IPackage):
     """An installed package"""
     def __init__(self, package_path):
@@ -112,6 +119,24 @@ class Package(IPackage):
             metadata = json.load(metadata_file)
             self.load_metadata_from_json(metadata)
 
+    def get_readme(self):
+        """Returns the text of the README.md in this package.
+
+        Returns:
+            (str): The text of the package README.md, None
+                if README.md can't be read
+
+        """
+        readme_path = self.package_path / 'README.md'
+        if not readme_path.is_file():
+            return None
+        with open(readme_path, 'r') as readme_file:
+            return readme_file.read()
+        return None
+
+    def get_description(self):
+        return self.get_readme()
+
 class AvailablePackage(IPackage):
     """A package available for download"""
     def __init__(self, metadata):
@@ -127,9 +152,11 @@ class AvailablePackage(IPackage):
         open(filepath, 'wb').write(r.content)
         return filepath
 
+    def get_description(self):
+        return "{} → {}".format(self.from_name, self.to_name)
 
 def install_from_path(path):
-    """Install a package (zip archive ending in .argosmodel).
+    """Install a package file (zip archive ending in .argosmodel).
 
     Args:
         path (str): The path to the .argosmodel file to install.
@@ -175,7 +202,7 @@ def update_package_index():
     r = requests.get(settings.remote_package_index, allow_redirects=True)
     open(settings.local_package_index, 'wb').write(r.content)
 
-def load_available_packages():
+def get_available_packages():
     """Returns a list of AvailablePackages from the package index."""
     try:
         with open(settings.local_package_index) as index_file:
@@ -189,3 +216,6 @@ def load_available_packages():
         raise Exception('Local package index not found,' +
                 ' use package.update_package_index() to load it')
 
+def load_available_packages():
+    """Deprecated 1.2, use get_available_packages"""
+    return get_available_packages()
