@@ -4,7 +4,92 @@ import ctranslate2
 import sentencepiece as spm
 import stanza
 
-from argostranslate import package
+from argostranslate import translate, package, settings, utils
+from argostranslate.utils import info, warn, error
+
+class Hypothesis:
+    """Represents a translation hypothesis
+
+    Attributes:
+        value (str): The hypothetical translation value
+        score (float): The score representing the quality of the translation
+    """
+
+    def __init__(self, value, score):
+        self.value = value
+        self.score = score
+
+    def __lt__(self, other):
+        return self.score < other.score
+
+    def __str__(self):
+        return "({}, {})".format(repr(self.value), self.score)
+
+    def __repr__(self):
+        return type(self).__name__ + self.__str__()
+
+class ITranslation:
+    """Respresents a translation between two Languages
+
+    Attributes:
+        from_lang (Language): The Language this Translation translates from.
+        to_lang (Language): The Language this Translation translates to.
+
+    """
+    def translate(self, input_text):
+        """Translates a string from self.from_lang to self.to_lang
+
+        Args:
+            input_text (str): The text to be translated.
+
+        Returns:
+            str: input_text translated.
+
+        """
+        return self.hypotheses(input_text, num_hypotheses=1)[0].value
+
+    def hypotheses(self, input_text, num_hypotheses=4):
+        """Translates a string from self.from_lang to self.to_lang
+
+        Args:
+            input_text (str): The text to be translated.
+            num_hypotheses (int): Number of hypothetic results expected
+
+        Returns:
+            [Hypothesis]: List of translation hypotheses
+
+        """
+        raise NotImplementedError()
+
+    @staticmethod
+    def split_into_paragraphs(input_text):
+        """Splits input_text into paragraphs and returns a list of paragraphs.
+
+        Args:
+            input_text (str): The text to be split.
+
+        Returns:
+            [str]: A list of paragraphs.
+
+        """
+        return input_text.split('\n')
+
+    @staticmethod
+    def combine_paragraphs(paragraphs):
+        """Combines a list of paragraphs together.
+
+        Args:
+            paragraphs ([str]): A list of paragraphs.
+            num_hypotheses (int): Number of hypothetic results to be combined
+
+        Returns:
+            [str]: list of n paragraphs combined into one string.
+
+        """
+        return '\n'.join(paragraphs)
+
+    def __str__(self):
+        return str(self.from_lang) + ' -> ' + str(self.to_lang)
 
 class Language:
     """Represents a language that can be translated from/to.
@@ -45,61 +130,8 @@ class Language:
             return valid_translations[0]
         return None
 
-class ITranslation:
-    """Respresents a translation between two Languages
-
-    Attributes:
-        from_lang (Language): The Language this Translation translates from.
-        to_lang (Language): The Language this Translation translates to.
-
-    """
-    def translate(self, input_text):
-        """Translates a string from self.from_lang to self.to_lang
-
-        Args:
-            input_text (str): The text to be translated.
-
-        Returns:
-            str: input_text translated.
-
-        """
-        raise NotImplementedError()
-
-    def split_into_paragraphs(self, input_text):
-        """Splits input_text into paragraphs and returns a list of paragraphs.
-
-        Args:
-            input_text (str): The text to be split.
-
-        Returns:
-            [str]: A list of paragraphs.
-
-        """
-        return input_text.split('\n')
-
-    def combine_paragraphs(self, paragraphs):
-        """Combines a list of paragraphs together.
-
-        Args:
-            paragraphs ([str]): A list of paragraphs.
-
-        Returns:
-            str: paragraphs combined into one string.
-
-        """
-        return '\n'.join(paragraphs)
-
-    def __str__(self):
-        return str(self.from_lang) + ' -> ' + str(self.to_lang)
-
 class PackageTranslation(ITranslation):
     """Translation from a package
-
-    Attributes:
-        from_lang (Language): The Language this Translation translates from.
-        to_lang (Language): The Language this Translation translates to.
-        pkg (Package): The installed package that provides this translation.
-
     """
 
     def __init__(self, from_lang, to_lang, pkg):
@@ -108,16 +140,31 @@ class PackageTranslation(ITranslation):
         self.pkg = pkg
         self.translator = None
 
-    def translate(self, input_text):
+    def hypotheses(self, input_text, num_hypotheses):
         if self.translator == None:
             model_path = str(self.pkg.package_path / 'model')
             self.translator = ctranslate2.Translator(model_path)
-        paragraphs = self.split_into_paragraphs(input_text)
+        paragraphs = ITranslation.split_into_paragraphs(input_text)
+        info("paragraphs", paragraphs)
         translated_paragraphs = []
         for paragraph in paragraphs:
             translated_paragraphs.append(
-                    apply_packaged_translation(self.pkg, paragraph, self.translator))
-        return self.combine_paragraphs(translated_paragraphs)
+                    apply_packaged_translation(self.pkg, paragraph, self.translator, num_hypotheses))
+        info("translated_paragraphs", translated_paragraphs)
+
+        # Construct new hypotheses using all paragraphs
+        hypotheses_to_return = [Hypothesis('', 0) for i in range(num_hypotheses)]
+        for i in range(num_hypotheses):
+            for translated_paragraph in translated_paragraphs:
+                value = ITranslation.combine_paragraphs([
+                        hypotheses_to_return[i].value,
+                        translated_paragraph[i].value
+                        ])
+                score = hypotheses_to_return[i].score + translated_paragraph[i].score
+                hypotheses_to_return[i] = Hypothesis(value, score)
+            hypotheses_to_return[i].value = hypotheses_to_return[i].value.lstrip('\n')
+        info('hypotheses_to_return', hypotheses_to_return)
+        return hypotheses_to_return
 
 class IdentityTranslation(ITranslation):
     """A Translation that doesn't modify input_text."""
@@ -133,8 +180,8 @@ class IdentityTranslation(ITranslation):
         self.from_lang = lang
         self.to_lang = lang
 
-    def translate(self, input_text):
-        return input_text
+    def hypotheses(self, input_text, num_hypotheses):
+        return [Hypothesis(input_text, 0) for i in range(num_hypotheses)]
 
 class CompositeTranslation(ITranslation):
     """A ITranslation that is performed by chaining two Translations
@@ -146,14 +193,37 @@ class CompositeTranslation(ITranslation):
     """
 
     def __init__(self, t1, t2):
+        """Creates a CompositeTranslation.
+
+        Args:
+            t1 (ITranslation): The first Translation to apply.
+            t2 (ITranslation): The second Translation to apply.
+
+        """
         self.t1 = t1
         self.t2 = t2
         self.from_lang = t1.from_lang
         self.to_lang = t2.to_lang
 
-    def translate(self, input_text):
-        return self.t2.translate(self.t1.translate(input_text))
+    def hypotheses(self, input_text, num_hypotheses):
+        t1_hypotheses = self.t1.hypotheses(input_text, num_hypotheses)
 
+        # Combine hypotheses
+        # O(n^2)
+        to_return = []
+        for t1_hypothesis in t1_hypotheses:
+            t2_hypotheses = self.t2.hypotheses(
+                    t1_hypothesis.value,
+                    num_hypotheses)
+            for t2_hypothesis in t2_hypotheses:
+                to_return.append(
+                        Hypothesis(
+                        t2_hypothesis.value,
+                        t1_hypothesis.score + t2_hypothesis.score \
+                        ))
+        to_return.sort()
+        return to_return[0:num_hypotheses]
+        
 class CachedTranslation(ITranslation):
     """Caches a translation to improve performance.
 
@@ -179,32 +249,48 @@ class CachedTranslation(ITranslation):
         self.to_lang = underlying.to_lang
         self.cache = dict()
 
-    def translate(self, input_text):
-        new_cache = dict()
-        paragraphs = self.split_into_paragraphs(input_text)
+    def hypotheses(self, input_text, num_hypotheses=4):
+        new_cache = dict() # 'text': ['t1'...('tN')]
+        paragraphs = ITranslation.split_into_paragraphs(input_text)
         translated_paragraphs = []
         for paragraph in paragraphs:
             translated_paragraph = self.cache.get(paragraph)
-            if translated_paragraph == None:
-                translated_paragraph = self.underlying.translate(paragraph)
+            # If len() of our cached items are different than `num_hypotheses` it means that
+            # the search parameter is changed by caller, so we can't re-use cache, and should update it.
+            if translated_paragraph == None or len(translated_paragraph) != num_hypotheses:
+                translated_paragraph = self.underlying.hypotheses(paragraph, num_hypotheses)
             new_cache[paragraph] = translated_paragraph
             translated_paragraphs.append(translated_paragraph)
         self.cache = new_cache
-        return self.combine_paragraphs(translated_paragraphs)
 
+        # Construct hypotheses
+        hypotheses_to_return = [Hypothesis('', 0) for i in range(num_hypotheses)]
+        for i in range(num_hypotheses):
+            for j in range(len(translated_paragraphs)):
+                value = ITranslation.combine_paragraphs([
+                        hypotheses_to_return[i].value,
+                        translated_paragraphs[j][i].value
+                        ])
+                score = hypotheses_to_return[i].score + translated_paragraphs[j][i].score
+                hypotheses_to_return[i] = Hypothesis(value, score)
+            hypotheses_to_return[i].value = hypotheses_to_return[i].value.lstrip('\n')
+        return hypotheses_to_return
 
-def apply_packaged_translation(pkg, input_text, translator): 
+def apply_packaged_translation(pkg, input_text, translator, num_hypotheses=4):
     """Applies the translation in pkg to translate input_text.
 
     Args:
         pkg (Package): The package that provides the translation.
         input_text (str): The text to be translated.
+        translator (ctranslate2.Translator): The CTranslate2 Translator
+        num_hypotheses (int): The number of hypotheses to generate
 
     Returns:
-        str: The translated text.
+        [Hypothesis]: A list of Hypothesis's for translating input_text
 
     """
 
+    info('apply_packaged_translation')
     sp_model_path = str(pkg.package_path / 'sentencepiece.model')
     sp_processor = spm.SentencePieceProcessor(model_file=sp_model_path)
     stanza_pipeline = stanza.Pipeline(lang=pkg.from_code,
@@ -213,27 +299,45 @@ def apply_packaged_translation(pkg, input_text, translator):
             logging_level='WARNING')
     stanza_sbd = stanza_pipeline(input_text)
     sentences = [sentence.text for sentence in stanza_sbd.sentences]
+    info('sentences', sentences)
     tokenized = [sp_processor.encode(sentence, out_type=str) for sentence in sentences]
+    info('tokenized', tokenized)
+    BATCH_SIZE = 32
+    assert(len(sentences) <= BATCH_SIZE)
     translated_batches = translator.translate_batch(
             tokenized,
             replace_unknowns=True,
-            max_batch_size=32,
+            max_batch_size=BATCH_SIZE,
+            beam_size=num_hypotheses,
+            num_hypotheses=num_hypotheses,
             length_penalty=0.2)
-    translated_tokens = []
-    for translated_batch in translated_batches:
-        translated_tokens += translated_batch[0]['tokens']
-    detokenized = ''.join(translated_tokens)
-    detokenized = detokenized.replace('▁', ' ')
-    to_return = detokenized
-    if len(to_return) > 0 and to_return[0] == ' ':
-        # Remove space at the beginning of the translation added
-        # by the tokenizer.
-        to_return = to_return[1:]
-    return to_return
+    info('translated_batches', translated_batches)
 
-def load_installed_languages():
+    # Build hypotheses
+    value_hypotheses = []
+    for i in range(num_hypotheses):
+        translated_tokens = []
+        cumulative_score = 0
+        for translated_batch in translated_batches:
+            translated_tokens += translated_batch[i]['tokens']
+            cumulative_score += translated_batch[i]['score']
+        detokenized = ''.join(translated_tokens)
+        detokenized = detokenized.replace('▁', ' ')
+        value = detokenized
+        if len(value) > 0 and value[0] == ' ':
+            # Remove space at the beginning of the translation added
+            # by the tokenizer.
+            value = value[1:]
+        hypothesis = Hypothesis(value, cumulative_score)
+        value_hypotheses.append(hypothesis)
+    info('value_hypotheses', value_hypotheses)
+    return value_hypotheses
+
+def get_installed_languages():
     """Returns a list of Languages installed from packages"""
     
+    info('get_installed_languages')
+
     packages = package.get_installed_packages()
 
     # Load languages and translations from packages
@@ -291,3 +395,6 @@ def load_installed_languages():
 
     return languages
 
+def load_installed_languages():
+    """Deprecated 1.2, use get_installed_languages"""
+    return get_installed_languages()
